/* Poniszy kod pochodzi z ksiki
* "C++. Programowanie zorientowane obiektowo.
* Vademecum profesjonalisty"
* autorstwa Nicolai M. Josuttisa, wydanej
* w Polsce przez wydawnictwo HELION S.A. w 2003 roku
* (wydanie oryginalne: Wiley, 2002)
*
* (C) Copyright Nicolai M. Josuttis 2002.
* Kopiowanie, wykorzystywanie, modyfikacja
* i rozpowszechnianie niniejszego oprogramowania
* jest dozwolone pod warunkiem zamieszczenia
* niniejszej informacji o prawach autorskich.
* Oprogramowanie to jest udostpniane
* bez jakichkolwiek gwarancji.
*/

#include <vector>
#include <stdexcept>

// **** Pocztek przestrzeni nazw CPPBook ********************************
namespace CPPBook {

template <typename T, typename CONT = std::vector<T> >
class Stack {
  private:
    CONT elems;    // elementy

  public:
    Stack();              // konstruktor
    void push(const T&);  // umieszcza nowy element na szczycie
    void pop();           // usuwa element ze szczytu
    T top() const;        // zwraca element znajdujcy si na szczycie
};

// konstruktor
template <typename T, typename CONT>
Stack<T,CONT>::Stack()
{
    // nie wykonuje adnych instrukcji
}

template <typename T, typename CONT>
void Stack<T,CONT>::push(const T& elem)
{
    elems.push_back(elem);    // umieszcza kopi na szczycie stosu
}

template <typename T, typename CONT>
void Stack<T,CONT>::pop()
{
    if (elems.empty()) {
        throw std::out_of_range("Stack<>::pop(): pusty stos");
    }
    elems.pop_back();         // usuwa element ze szczytu stosu
}

template <typename T, typename CONT>
T Stack<T,CONT>::top() const
{
    if (elems.empty()) {
        throw std::out_of_range("Stack<>::top(): pusty stos");
    }
    return elems.back();      // zwraca kopi elementu znajdujcego si na szczycie
}

} // **** Koniec przestrzeni nazw CPPBook ********************************

